;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xd89984874a6d9a1548cd980028288947 . "./code.cg2js/cg-acache.fasl")
(#xa7ff19f93f3054260ff0b8e8ee160d64 . "./code.cg2js/cg-base.fasl")
(#xf79dc4b19581facd0f825b3220a49d6b . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x0ecfedd6fbbb293171ddba748bccfe31 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7f17351834b77a78997dfd9444efc12c . "./code.cg2js/cg-bitmap-stream.fasl")
(#xdb409809cd1bcf076c6bc424a716f3e0 . "./code.cg2js/cg-button.fasl")
(#x8a0c6ef27a3ae6c292d190b81cb99e24 . "./code.cg2js/cg-calendar.fasl")
(#x5c68c72dafcd5b740f46d94f69fc9bed . "./code.cg2js/cg-caret.fasl")
(#x5601e43662c1977845a1766254d8ea5c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xb5377ca7cde148050b0dafb6339c2689 . "./code.cg2js/cg-chart-widget.fasl")
(#xb2213e1c35e7f1eaba079b33329d4f7f . "./code.cg2js/cg-check-box.fasl")
(#xe374412ca8b22300f237a941d4373033 . "./code.cg2js/cg-choice-list.fasl")
(#x85b9f0e43ad5c42d9b6c9a1b6b0bbfcf . "./code.cg2js/cg-choose-printer.fasl")
(#x338f955a44d52042ed268663dc3c33ae . "./code.cg2js/cg-class-grid.fasl")
(#xe6e9676b1dd6b91802041bd7285b1162 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x2e52b84641d236a7c50b24392c27a1f7 . "./code.cg2js/cg-class-support.fasl")
(#xd448f17f8b778851a0a48bdea64c2bc0 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xafee2138529fd6401da2a9d1e947e9a7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xe0603430dac5a17b05578ab96235820b . "./code.cg2js/cg-clipboard.fasl")
(#xb0e3d7247f44e7b1350e9bdc895d6f0a . "./code.cg2js/cg-color-dialog.fasl")
(#x8fd3e5d9dc8b11d74ef3aef6464b78b3 . "./code.cg2js/cg-combo-box.fasl")
(#x536dce7289b05167092b556e30aa2255 . "./code.cg2js/cg-common-control.fasl")
(#x4ecd32fe60701d6d342ddab6829bbc3e . "./code.cg2js/cg-comtab.fasl")
(#xe0436040911b0d7f4527a8a2f87c5808 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x550a5d71205bab3279d7525d0312ccc5 . "./code.cg2js/cg-curve.fasl")
(#xf1c2d95a2aa2c81872baf9255d3bc395 . "./code.cg2js/cg-dde-utils.fasl")
(#xafb935db9f1770000d675355cdca79ac . "./code.cg2js/cg-dev.fasl")
(#xfeba297aebe0cd8985546ae6df8e5343 . "./code.cg2js/cg-dialog-item.fasl")
(#xad3493f283af2ac517c99736617545e7 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x9deb553fbbe508279f3be570d0236425 . "./code.cg2js/cg-directory-dialog.fasl")
(#xda10545441f5ba9461686726c5f3c500 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0d8924b9f01a6dec43466bb9f796dcef . "./code.cg2js/cg-drag-and-drop.fasl")
(#xc01c1980b593d0e442a05140f25a46b4 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x2e48edd9e5dc99cade6fd677044b428b . "./code.cg2js/cg-drawable.fasl")
(#x390adb620f501c5762281bfbe46be413 . "./code.cg2js/cg-dropping-outline.fasl")
(#xd18722fe9aa31f40b18db20d578a7a10 . "./code.cg2js/cg-edit-in-place.fasl")
(#x583d1035da22c9b6d5e8ef50d2fa7fea . "./code.cg2js/cg-editable-text.fasl")
(#x70179f2eab8a10a8ec7ab3b13ad65e73 . "./code.cg2js/cg-file-dialog.fasl")
(#xba11cfd5f26617a2f23962e8d2a8b7d0 . "./code.cg2js/cg-fill-texture.fasl")
(#xfc7c68ed972da69b622b6c7466eb9cc5 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xa38e9de6d8c791bf69a39dc410bf5955 . "./code.cg2js/cg-font-dialog.fasl")
(#xbc4c5aca17af1c0a044c6e6df0da647a . "./code.cg2js/cg-gesture-emulation.fasl")
(#x1f860b1dd671b910ec5a2df6995b3609 . "./code.cg2js/cg-get-pixmap.fasl")
(#x6d11db62ea5a86cdeaf624cadfdff562 . "./code.cg2js/cg-get-position.fasl")
(#xc20ad3e62553077f90207f4cc6e6c3c6 . "./code.cg2js/cg-graphics-context.fasl")
(#x68a0752424b743a38c0bcdba4fc38b14 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x894db98c6b5432f9e4d38118304917e2 . "./code.cg2js/cg-grid-widget.fasl")
(#xcbca27790671bafbd1769e385e5b78c3 . "./code.cg2js/cg-group-box.fasl")
(#x049259a0f58d6601bd587662618cde2d . "./code.cg2js/cg-header-control.fasl")
(#x071f1dbe7e6ade74fac9455aa04a73ff . "./code.cg2js/cg-hotspot.fasl")
(#x6edbf7edb7d8a6f31b373b4d6cdf098c . "./code.cg2js/cg-html-dialog.fasl")
(#xda8c69dd339d128bbcd6d66dc19eb0fb . "./code.cg2js/cg-html-widget.fasl")
(#x8a695f4346db30031fd8e08e734d590c . "./code.cg2js/cg-icon-pixmap.fasl")
(#xb0561012dec96d684e19d3e5366436d5 . "./code.cg2js/cg-icon.fasl")
(#x60e2ed650811776b8cbb91303d0b52cd . "./code.cg2js/cg-ie.fasl")
(#x20f31fc2edff5d0ba2dc8d1adf8573c4 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x833967fd3ca14543a53bf2b1d691991e . "./code.cg2js/cg-intersect.fasl")
(#x4b014235e4e1fbd8e269494b25db083b . "./code.cg2js/cg-item-list.fasl")
(#xfa7a0dffac84908bc1f26702e729cd3d . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x251cc5cbaa998d26ff5627bf9fc64a1c . "./code.cg2js/cg-lamp.fasl")
(#x4852eab0853366e46f4ec524aed47065 . "./code.cg2js/cg-layout.fasl")
(#x7a59ef7713fbdcd720eba8fb73894dad . "./code.cg2js/cg-lettered-menu.fasl")
(#xa77640c63333fdb4ae1f5b8897b0daa9 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9b96b5ade5a6bf2d484ff843014d5f00 . "./code.cg2js/cg-lisp-text.fasl")
(#xf6618ff21facaf398597fc5b3c314a14 . "./code.cg2js/cg-lisp-widget.fasl")
(#x10523a10d5d1c1fe5414a510d95554d4 . "./code.cg2js/cg-list-view.fasl")
(#xb3b9386922e81e1c6dc123da5d5f0ef9 . "./code.cg2js/cg-mci.fasl")
(#xeac21591e97d858cd070043e34ace757 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x2931dd7fbb352c1a51893d60f454eeec . "./code.cg2js/cg-menu.fasl")
(#xc517ad0d638cc3985c167fb7f0dc8a44 . "./code.cg2js/cg-message-dialog.fasl")
(#xad520be9de8e38830d1c75be45f828b0 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xb68503bb8b90ae24e5d299f7fdeab55f . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x38e9a76f6018c44554300d002e3a12ef . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x9d2192cfddc3b10f45dccdbf89889936 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x3f459b24072fd18aa168c33d98791ca0 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x71cfeb045166e7c2f6b78c46f00b45a9 . "./code.cg2js/cg-nodes.fasl")
(#xe26dd197fca4581cf6f87476a1b25acd . "./code.cg2js/cg-object-editor-layout.fasl")
(#x5419e646a0510704ab4c839e86a08ce1 . "./code.cg2js/cg-object-editor.fasl")
(#x66d5b0a4b755847f6f97131225430bbd . "./code.cg2js/cg-ocx.fasl")
(#x71e2530fa385f8959bf809496521a088 . "./code.cg2js/cg-os-widget.fasl")
(#x5ddf9ed4c9a62fb9ac59507ad63872c5 . "./code.cg2js/cg-os-window.fasl")
(#x3a066fa4152df3e426321d9570088191 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x6cd950f720f3e6fb8902cc04bf2c42af . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xa6bc33aaaa43b53ad264e20f1b2dce54 . "./code.cg2js/cg-outline.fasl")
(#xcb7af7a854314fc75c15a5be4294453f . "./code.cg2js/cg-palette.fasl")
(#x92358009c37ba75be9b71a38b5db49ef . "./code.cg2js/cg-paren-matching.fasl")
(#x09c1640e2957f2f06cec551d3307c3f4 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xa2f10c97ea5c57aa04ab56f3ce99fec1 . "./code.cg2js/cg-picture-widget.fasl")
(#xc119951b9c562a8c8cb970b84466d2bb . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x62d5dfd8f424f9fbb356a4c2512993c4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd8f917ab35f5a3e5658c0d3277044966 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xeb50a7e7ed1eba2bc7ee78dad833ce55 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xcb2cb5b433df900885ccdc289fb0430b . "./code.cg2js/cg-pixmap.fasl")
(#xcd654e83a8e1a52fd9455bcc992edf9b . "./code.cg2js/cg-plot-widget.fasl")
(#xf65ff41dd6818ec3697c5b99276bf643 . "./code.cg2js/cg-printing.fasl")
(#x9b2e8a9d7c1a1291e525ac5545a8ce24 . "./code.cg2js/cg-progress-indicator.fasl")
(#xd02b824eb90f6c2e0229997767be7046 . "./code.cg2js/cg-project-window.fasl")
(#xad5204709712c6ad959dc934971ac498 . "./code.cg2js/cg-property.fasl")
(#x89678000e67b539ae727da62b1cf95d2 . "./code.cg2js/cg-radio-button.fasl")
(#x8270c7c988a10b824805b888dd475ce7 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xa25687df2d9c1a53af01fde045ee02f3 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xbbae331a345c0063ada998d16918a675 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x5f0eafe38b9d9c7837c0c3bf3a9115e1 . "./code.cg2js/cg-rich-edit.fasl")
(#x577a2bf682611afc0d56f06e8c896b7e . "./code.cg2js/cg-sample-file-menu.fasl")
(#xeacc1d55062778b5548e9b00c094c621 . "./code.cg2js/cg-scaling-stream.fasl")
(#x851b83c99908ebfacac77f06bce985d3 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x6869cbd3e77f8f79ef73028a9198b351 . "./code.cg2js/cg-scroll-bar.fasl")
(#xa77782f460b33e4a268424c7b3e6e1c5 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x86331944b792936cceec4d217cd0ee9f . "./code.cg2js/cg-selected-object.fasl")
(#x4527595edaad7807dfcc753f1f1561da . "./code.cg2js/cg-shortcut-menu.fasl")
(#x2db0656e6cbe11cc4d3e7d122f6305aa . "./code.cg2js/cg-split-bar.fasl")
(#x4ce0a2dfc3f262d3c0a4345128af1762 . "./code.cg2js/cg-static-text.fasl")
(#x0b7549fe336e82d8cde9747101a7d1e0 . "./code.cg2js/cg-status-bar.fasl")
(#x034a5cee49c8db956ec0c8daaa0f829c . "./code.cg2js/cg-string-dialog.fasl")
(#xd1e9f987cd6db94059b9496f9a3d9070 . "./code.cg2js/cg-tab-control.fasl")
(#x4d7a50879f0bf9d9fc3486d82b1d88a7 . "./code.cg2js/cg-template-string.fasl")
(#x6d87767671ea9fdf6c1e32044dfcaad5 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xc7604cc43801e11d351e1e059f8c03cf . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x91a9b0c0e10bc909a3bc0b419238eecf . "./code.cg2js/cg-text-edit-pane.fasl")
(#xf2dc9bfe91f30e235cc54107dbf8b8ac . "./code.cg2js/cg-text-or-combo.fasl")
(#x233e3a2055ea3da264f1fc11ada49260 . "./code.cg2js/cg-text-widget.fasl")
(#x4dde36d75100819215acbc2ee146ea35 . "./code.cg2js/cg-timer.fasl")
(#xe0d5c658468ec9c68974c9e67168fd48 . "./code.cg2js/cg-toggling-widget.fasl")
(#x9d7e93c07fdc9cb379fee46103558df3 . "./code.cg2js/cg-toolbar.fasl")
(#x24e3fd884a84ffa539709cd3788a1b41 . "./code.cg2js/cg-tooltip.fasl")
(#xd5e09eae378ddfdc5600f553fb8173a9 . "./code.cg2js/cg-touch.fasl")
(#x82d55f34de006997d715a4e4e7a34dd7 . "./code.cg2js/cg-trackbar.fasl")
(#x21f10fc84b97b5a3d874c05423af8151 . "./code.cg2js/cg-tray.fasl")
(#x2e0af093085a9b0c5bb5b4dfb7d16d45 . "./code.cg2js/cg-up-down-control.fasl")
(#x333b72aacd7065be4fec25f91eb8e34f . "./code.cg2js/cg-utility-dialog.fasl")
(#x2b2221971258e82d785b431541324aa3 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x80736a9f5b01028bac9a782730e16f91 . "./code.cg2js/cg-web-browser.fasl")
(#xd4c830b9d4b4d7c02d518abb21f31dd5 . "./code.cg2js/cg-wrap-string.fasl")
(#x9aa443c05783308c8025fd2a477894db . "./code.cg2js/cg-yes-no-list.fasl")
(#xf9e9e1b301c574064dc8ffda0075cdec . "./code.cg2js/cg-yes-no-string.fasl")
(#xd332db83fc8235c003a74af34b098f99 . "./code.cg2js/cg.fasl")
(#x07dfd52bf1ed3179d936ec5d48df5442 . "./code.idejs/ide-acache.fasl")
(#x023a5b120e961e4f4e5f79c841cf1b2a . "./code.idejs/ide-apropos.fasl")
(#x8881d38a05c4b5eaa5f2ee48aa558e15 . "./code.idejs/ide-base.fasl")
(#xaf4adc03168202cb86a77015da2e5c39 . "./code.idejs/ide-class-browser.fasl")
(#x3cc8d917bb054076aec93bd8a68000c0 . "./code.idejs/ide-class-grapher.fasl")
(#xdcaac65ccd45cecc25a2d0364ec7ddfb . "./code.idejs/ide-clipboard.fasl")
(#xd63b5f410d22883764b8909547560686 . "./code.idejs/ide-dev.fasl")
(#x106694d59f26f53f7e1bc4a4f4bc8b61 . "./code.idejs/ide-editor.fasl")
(#xf1336c3c95b896a5eda2d16617f7ce4d . "./code.idejs/ide-find-definitions.fasl")
(#xa249000bb26fb4c80e645b3f9ad3249d . "./code.idejs/ide-find-in-files.fasl")
(#x78bcffbb33a8955cc33f258482efdcfb . "./code.idejs/ide-form.fasl")
(#xd7b7a8c81e78a4aafe63feda5df01e05 . "./code.idejs/ide-grapher.fasl")
(#xed8243e2a41dd5e9f50c3313d3b07bb4 . "./code.idejs/ide-inspector.fasl")
(#x35ca788e32a0fd46730bdfbf4bf54d51 . "./code.idejs/ide-intro.fasl")
(#x7c035ae255ee5801614715855c045800 . "./code.idejs/ide-keyshort.fasl")
(#xa63a4626cb4898c0ff6ba20273662a47 . "./code.idejs/ide-navigator.fasl")
(#x254f3d7578dafa19fa277795db069e85 . "./code.idejs/ide-package-list.fasl")
(#xa19211bd750cc08de1cf0409cac52dc7 . "./code.idejs/ide-patch-dialog.fasl")
(#xd305c461b258e5db069f834180319f9e . "./code.idejs/ide-process-browser.fasl")
(#x669d4c9cf1d4bac35fa6f567fd7c6844 . "./code.idejs/ide-project.fasl")
(#x3d73755ee1f3aba90cb4deabd9c28873 . "./code.idejs/ide-trace.fasl")
(#x4c4a77e3644e5cef1be9f9b25b208dbc . "./code.idejs/ide-window-list.fasl")
(#x0c66b08391e19ce2a05eee25119dcc08 . "./code.idejs/ide.fasl")
(#x7f480facc219636edfe865d5e108e9cb . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
